'''
Copyright (C) 2024 1P2D
https://1p2d.com
me@1p2d.com

Created by 1P2D - Felix Pierron

This file is part of the Shading + pie.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, see <https://www.gnu.org/licenses/>.
'''

import bpy
from bpy.types import Operator


class ToggleCompositor(bpy.types.Operator):
    bl_idname = "shading_plus.toggle_compositor"
    bl_label = "Toggle Compositor"
    bl_description = "Toggle the compositor's use of nodes on or off"

    def execute(self, context):
        scene = context.scene
        compositor_active = scene.use_nodes if scene else False

        # Toggle compositor based on current state
        if compositor_active:
            scene.use_nodes = False
            self.report({'INFO'}, "Compositor nodes turned off")
        else:
            scene.use_nodes = True
            self.report({'INFO'}, "Compositor nodes turned on")

        return {'FINISHED'}



class AddBackground(bpy.types.Operator):
    """Add Background"""
    bl_idname = "shading_plus.add_background"
    bl_label = "Add Background"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        scene = context.scene
        space = context.space_data

        # Ensure the active space is a 3D Viewport
        if space.type != 'VIEW_3D':
            self.report({'WARNING'}, "Active space is not a 3D Viewport.")
            return {'CANCELLED'}

        # Remember the current shading mode
        previous_shading = space.shading.type

        try:
            # Switch to RENDERED shading mode if not already
            if space.shading.type != 'RENDERED':
                space.shading.type = 'RENDERED'

            # Enable film transparency and compositor nodes
            scene.render.film_transparent = True
            scene.use_nodes = True

            # Get the compositor node tree
            node_tree = scene.node_tree

            # Ensure there is a Render Layers node
            render_layers = None
            for node in node_tree.nodes:
                if node.type == 'R_LAYERS':
                    render_layers = node
                    break

            if not render_layers:
                # Add a Render Layers node if it doesn't exist
                render_layers = node_tree.nodes.new(type='CompositorNodeRLayers')
                render_layers.location = 0, 0
                self.report({'INFO'}, "Added Render Layers node to the compositor.")

            # Check if an Alpha Over node already exists to prevent duplicates
            alpha_over = None
            for node in node_tree.nodes:
                if node.type == 'ALPHAOVER' and node.name == "Alpha Over":
                    alpha_over = node
                    break

            if not alpha_over:
                # Add an Alpha Over node
                alpha_over = node_tree.nodes.new(type='CompositorNodeAlphaOver')
                alpha_over.location = render_layers.location.x + 200, render_layers.location.y
                alpha_over.name = "Alpha Over"
                self.report({'INFO'}, "Added Alpha Over node to the compositor.")
            else:
                self.report({'INFO'}, "Alpha Over node already exists.")

            # Debugging: Print input socket names and indices
            # Uncomment the following lines to debug socket names
            # for input_socket in alpha_over.inputs:
            #     print(f"Alpha Over Input Socket: {input_socket.name}, Type: {input_socket.type}, Index: {input_socket.index}")

            # Link Render Layers 'Image' output to Alpha Over's SECOND image input ('Image.001')
            try:
                # Remove existing links to prevent duplication
                links_to_alpha_over_second = [
                    link for link in node_tree.links 
                    if link.to_node == alpha_over and link.to_socket == alpha_over.inputs.get('Image.001')
                ]
                for link in links_to_alpha_over_second:
                    node_tree.links.remove(link)


                # Connect using the socket name 'Image.001'
                node_tree.links.new(render_layers.outputs['Image'], node_tree.nodes["Alpha Over"].inputs[2])
                self.report({'INFO'}, "Linked Render Layers 'Image' to Alpha Over's second image input ('Image.001').")
            except KeyError:
                self.report({'ERROR'}, "Alpha Over node does not have an 'Image.001' input.")
                # Revert shading mode
                space.shading.type = previous_shading
                return {'CANCELLED'}

            # Link Render Layers 'Alpha' output to Alpha Over's 'Fac' input
            try:
                # Remove existing links to prevent duplication
                links_to_alpha_over_fac = [
                    link for link in node_tree.links 
                    if link.to_node == alpha_over and link.to_socket == alpha_over.inputs['Fac']
                ]
                for link in links_to_alpha_over_fac:
                    node_tree.links.remove(link)

                node_tree.links.new(render_layers.outputs['Alpha'], alpha_over.inputs['Fac'])
                self.report({'INFO'}, "Linked Render Layers 'Alpha' to Alpha Over's 'Fac' input.")
            except KeyError:
                self.report({'ERROR'}, "Render Layers node does not have an 'Alpha' output.")
                # Revert shading mode
                space.shading.type = previous_shading
                return {'CANCELLED'}

            # Check if a Viewer node already exists
            viewer = None
            for node in node_tree.nodes:
                if node.type == 'VIEWER' and node.name == "Viewer":
                    viewer = node
                    break

            if not viewer:
                # Add a Viewer node
                viewer = node_tree.nodes.new(type='CompositorNodeViewer')
                viewer.location = alpha_over.location.x + 200, alpha_over.location.y
                viewer.name = "Viewer"
                self.report({'INFO'}, "Added Viewer node to the compositor.")
            else:
                self.report({'INFO'}, "Viewer node already exists.")

            # Link Alpha Over 'Image' output to Viewer 'Image' input
            try:
                # Remove existing links to prevent duplication
                links_to_viewer = [
                    link for link in node_tree.links 
                    if link.from_node == alpha_over 
                    and link.from_socket == alpha_over.outputs['Image'] 
                    and link.to_node == viewer 
                    and link.to_socket == viewer.inputs['Image']
                ]
                for link in links_to_viewer:
                    node_tree.links.remove(link)

                node_tree.links.new(alpha_over.outputs['Image'], viewer.inputs['Image'])
                self.report({'INFO'}, "Linked Alpha Over's 'Image' output to Viewer's 'Image' input.")
            except KeyError:
                self.report({'ERROR'}, "Alpha Over node does not have an 'Image' output.")
                # Revert shading mode
                space.shading.type = previous_shading
                return {'CANCELLED'}

            # Set compositor to always use
            space.shading.use_compositor = 'ALWAYS'

            # Revert to the previous shading mode
            space.shading.type = previous_shading

            # Display info message
            self.report({'INFO'}, "Background color added - switch to EEVEE, Cycles or Workbench to see")

            return {'FINISHED'}

        except Exception as e:
            self.report({'ERROR'}, f"An unexpected error occurred: {e}")
            # Revert shading mode
            space.shading.type = previous_shading
            return {'CANCELLED'}



# REGISTER/UNREGISTER
# __________________________________________________________________

classes = (
    ToggleCompositor,
    AddBackground,
)

def register():
    for cls in classes:
        try:
            bpy.utils.unregister_class(cls)
        except RuntimeError:
            pass
        bpy.utils.register_class(cls)
    
def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

    
if __name__ == "__main__":
    register()





