'''
Copyright (C) 2024 1P2D
https://1p2d.com
me@1p2d.com

Created by 1P2D - Felix Pierron

This file is part of the Shading + pie.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, see <https://www.gnu.org/licenses/>.
'''

import bpy
from bpy.props import EnumProperty, PointerProperty, BoolProperty, StringProperty
from bpy.types import Operator, PropertyGroup
from bpy.utils import previews
import os

# Define the list of items directly
preset_items = [
    ('DEFAULT', "Blender Default", "Blender's default shading settings"),
    ('CELL', "Cell Shading", "Settings to mimic a cell-shading aesthetics"),
    ('ALIAS1', "Alias Shade Sky", "Replicate Alias Shade-Sky shader"),
    ('ALIAS2', "Alias Showroom", "Replicate Alias Showroom shader"),
    ('ALIAS3', "Alias Photo Horizon", "Replicate Alias Photo Horizon shader"),
    ('ALIAS4', "Alias Double Horizon", "Replicate Alias Double Horizon shader"),
    ('RED1', "Red Matcap 1", "A red reflective matcap ideal for hard surface modelling"),
    ('RED2', "Red Matcap 2", "A red reflective matcap ideal for hard surface modelling"),
    ('FUEL', "Fuel Shaded", "A purple reflective matcap ideal for hard surface modelling"),
    ('USER1', "User Preset 1", "Custom shading settings stored by the user"),  # New Preset Added
]



# **1. Define apply_preset_to_shading function**
def apply_preset_to_shading(preset, shading, shading_props=None):
    if preset == 'DEFAULT':
        shading.type = 'SOLID'
        shading.light = 'STUDIO'
        shading.color_type = 'SINGLE'
        shading.show_shadows = True
        shading.show_cavity = True
        shading.cavity_type = 'BOTH'
        shading.background_type = 'WORLD'
        shading.show_object_outline = False

    elif preset == 'CELL':
        shading.type = 'SOLID'
        shading.light = 'FLAT'
        shading.color_type = 'MATERIAL'        
        shading.show_shadows = True
        shading.show_cavity = True
        shading.cavity_type = 'BOTH'
        shading.background_type = 'VIEWPORT'  # Use viewport shading background
        shading.background_color = (1, 1, 1)  # Set the background color to white
        shading.show_object_outline = True

    elif preset == 'ALIAS1':
        shading.type = 'SOLID'
        shading.color_type = 'SINGLE'
        shading.single_color = (1, 1, 1)
        shading.background_type = 'THEME'
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Alias - Shade Sky.exr'
        shading.show_cavity = False
        shading.show_shadows = False
        shading.show_object_outline = False

    elif preset == 'ALIAS2':
        shading.type = 'SOLID'
        shading.background_type = 'THEME'
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Alias - Showroom.exr'
        shading.show_cavity = False
        shading.show_shadows = False
        shading.show_object_outline = False

    elif preset == 'ALIAS3':
        shading.type = 'SOLID'
        shading.background_type = 'THEME'
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Alias - Photo Horizon.exr'
        shading.show_cavity = False
        shading.show_shadows = False
        shading.show_object_outline = False

    elif preset == 'ALIAS4':
        shading.type = 'SOLID'
        shading.background_type = 'THEME'
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Alias - Double Horizon.exr'
        shading.show_cavity = False
        shading.show_shadows = False
        shading.show_object_outline = False

    elif preset == 'FUEL':
        shading.type = 'SOLID'
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Misc - Fuel Shaded.exr'
        shading.show_shadows = True
        shading.show_cavity = True
        shading.cavity_type = 'BOTH'
        shading.background_type = 'WORLD'
        shading.show_object_outline = True

    elif preset == 'RED1':
        shading.type = 'SOLID'
        shading.background_type = 'THEME'
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Misc - Red Gloss Better.exr'
        shading.show_cavity = False
        shading.show_shadows = False
        shading.show_object_outline = False

    elif preset == 'RED2':
        shading.type = 'SOLID'
        shading.background_type = 'THEME'        
        shading.light = 'MATCAP'
        shading.color_type = 'SINGLE'
        shading.studio_light = 'Misc - Red Gloss Old Blender.exr'
        shading.show_cavity = False
        shading.show_shadows = False
        shading.show_object_outline = False

    elif preset == 'USER1':
        if shading_props is None:
            print("Error: shading_props is required for USER1 preset.")
            return
        # Apply User Preset 1 Settings from stored data
        shading.type = shading_props.user1_settings.type
        shading.light = shading_props.user1_settings.light
        shading.studio_light = shading_props.user1_settings.studio_light
        shading.color_type = shading_props.user1_settings.color_type
        shading.show_shadows = shading_props.user1_settings.show_shadows
        shading.show_cavity = shading_props.user1_settings.show_cavity
        shading.cavity_type = shading_props.user1_settings.cavity_type
        shading.background_type = shading_props.user1_settings.background_type
        shading.show_object_outline = shading_props.user1_settings.show_object_outline

    else:
        # Fallback to Default if unknown preset
        shading.type = 'SOLID'



# Define update_shading_preset using apply_preset_to_shading
def update_shading_preset(self, context):

    preset = getattr(self, 'shading_preset', None)
    if preset is None:
        return

    # Ensure 'context' is valid
    if context is None or not hasattr(context, 'area') or context.area is None:
        return

    area = context.area
    if area.type != 'VIEW_3D':
        return

    space = area.spaces.active
    if space is None:
        return

    print(f"Selected Shading Preset: {preset}")

    # Access shading_props
    shading_props = self  # 'self' refers to ShadingPlusProperties

    # Apply the preset to the viewport shading
    apply_preset_to_shading(preset, space.shading, shading_props)

    # Redraw the 3D view to apply changes
    bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)







def update_builtin_hdri(self, context):
    area = context.area
    if area and area.type == 'VIEW_3D':
        space = context.space_data
        space.shading.studio_light = self.built_in_hdri
    else:
        pass  # Do nothing if not in 3D view




class SaveUser1Preset(Operator):
    bl_idname = "shading_plus.save_user_preset"
    bl_label = "Save Shading as USER1"
    bl_description = "Save current viewport shading settings to USER1 preset"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        shading = context.space_data.shading
        shading_props = context.scene.shading_plus

        # Store the current shading settings
        shading_props.user1_settings.type = shading.type
        shading_props.user1_settings.light = shading.light
        shading_props.user1_settings.studio_light = shading.studio_light  # Corrected line
        shading_props.user1_settings.wireframe_color_type = shading.wireframe_color_type
        shading_props.user1_settings.color_type = shading.color_type
        shading_props.user1_settings.show_shadows = shading.show_shadows
        shading_props.user1_settings.show_cavity = shading.show_cavity
        shading_props.user1_settings.cavity_type = shading.cavity_type
        shading_props.user1_settings.background_type = shading.background_type
        shading_props.user1_settings.show_object_outline = shading.show_object_outline

        # **Store 'studio_light' if applicable**
        if shading.light in {'STUDIO', 'MATCAP'}:
            shading_props.user1_settings.studio_light = shading.studio_light
            print(f"Saved studio_light: {shading.studio_light}")
        else:
            shading_props.user1_settings.studio_light = ""
            print("Cleared studio_light as lighting type does not require it.")

        self.report({'INFO'}, "USER1 shading preset saved successfully.")
        return {'FINISHED'}


class UserShadingSettings(PropertyGroup):

    type: EnumProperty(
        name="Shading Type",
        description="Viewport shading type",
        items=[
            ('WIREFRAME', "Wireframe", ""),
            ('SOLID', "Solid", ""),
            ('MATERIAL', "Material", ""),
            ('RENDERED', "Rendered", ""),
        ],
        default='SOLID'
    )

    light: EnumProperty(
        name="Lighting",
        description="Viewport lighting type",
        items=[
            ('FLAT', "Flat", ""),
            ('STUDIO', "Studio", ""),
            ('MATCAP', "Matcap", ""),
        ],
        default='STUDIO'
    )


    studio_light: StringProperty(
        name="Studio Light",
        description="Studio light used for shading (Matcap name)",
        default=""
    )

    wireframe_color_type: EnumProperty(
        name="Wireframe Color Type",
        description="Wireframe Color Type",
        items=[
            ('THEME', "Theme", ""),
            ('OBJECT', "Object", ""),
            ('RANDOM', "Random", ""),
        ],
        default='THEME'
    )

 
    color_type: EnumProperty(
        name="Color Type",
        description="Color type for shading",
        items=[
            ('SINGLE', "Single", ""),
            ('MATERIAL', "Material", ""),
            ('OBJECT', "Object", ""),
            ('RANDOM', "Random", ""),
            ('TEXTURE', "Texture", ""),
            ('VERTEX', "Attribute", ""),
        ],
        default='SINGLE'
    )
    show_shadows: BoolProperty(
        name="Show Shadows",
        description="Display shadows",
        default=True
    )
    show_cavity: BoolProperty(
        name="Show Cavity",
        description="Display cavity",
        default=True
    )
    cavity_type: EnumProperty(
        name="Cavity Type",
        description="Type of cavity",
        items=[
            ('BOTH', "Both", ""),
            ('WORLD', "World", ""),
            ('SCREEN', "Screen", ""),
        ],
        default='BOTH'
    )
    background_type: EnumProperty(
        name="Background Type",
        description="Background type for shading",
        items=[
            ('WORLD', "World", ""),
            ('VIEWPORT', "Viewport", ""),
            ('THEME', "Theme", ""),
        ],
        default='WORLD'
    )

    use_dof: BoolProperty(
        name="Use Depth of Field",
        description="Use Depth of Field",
        default=False
    )

    show_object_outline: BoolProperty(
        name="Show Object Outline",
        description="Display object outline",
        default=False
    )
    

class ShadingPlusProperties(PropertyGroup):
    shading_preset: EnumProperty(
        name="Shading Preset",
        description="Shading preset applied to Solid Rendered/Workbench shading",
        items=preset_items,
        default='DEFAULT',
        update=update_shading_preset,
    )

    user1_settings: PointerProperty(
        name="User Shading Settings",
        type=UserShadingSettings,
        description="Custom shading settings for USER1 preset"
    )

    built_in_hdri: EnumProperty(
        name="Built-in HDRI",
        description="Select built-in HDRI",
        items=[
            ('city.exr', "", "City HDRI"),
            ('courtyard.exr', "", "Courtyard HDRI"),
            ('forest.exr', "", "Forest HDRI"),
            ('interior.exr', "", "Interior HDRI"),
            ('night.exr', "", "Night HDRI"),
            ('studio.exr', "", "Studio HDRI"),
            ('sunrise.exr', "", "Sunrise HDRI"),
            ('sunset.exr', "", "Sunset HDRI"),
        ],
        default='forest.exr',
        update=update_builtin_hdri,
    )

# **6. Define BaseOperator**
class BaseOperator(bpy.types.Operator):
    bl_idname = ""
    bl_label = ""

    def execute(self, context):
        self.set_shading_settings(context)
        return {'FINISHED'}

    def set_shading_settings(self, context):
        pass

# **7. Define Existing Shading Operators**
class SetEeveeRenderedView(bpy.types.Operator):
    """Switch to Rendered Shading and set the Render Engine to EEVEE/EEVEE Next with optional max settings"""
    bl_idname = "shading_plus.set_eevee_rendered_view"
    bl_label = "Eevee Rendered View"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        area = context.area
        if area and area.type == 'VIEW_3D':
            space = area.spaces.active
            # Get Blender version
            blender_version = bpy.app.version

            # Set Render Engine based on Blender version
            if blender_version >= (4, 1, 0):
                # For Blender 4.1 and above, use EEVEE Next
                context.scene.render.engine = 'BLENDER_EEVEE_NEXT'
            else:
                # For Blender versions below 4.1, use EEVEE
                context.scene.render.engine = 'BLENDER_EEVEE'


            # Set Viewport Shading to Rendered
            space.shading.type = 'RENDERED'
            if blender_version >= (4, 1, 0):
                self.report({'INFO'}, "Switched to Rendered Shading: Eevee Next")
            else:
                self.report({'INFO'}, "Switched to Rendered Shading: Eevee")

            # Access Add-on Preferences
            addon_prefs = context.preferences.addons.get(__package__)
            if addon_prefs:
                prefs = addon_prefs.preferences
            else:
                prefs = None

            # Apply Additional Settings Only for Blender 4.1+ and if option enabled in Preferences
            if blender_version >= (4, 1, 0) and prefs and prefs.use_eevee_next_optimized_settings:
                # Ensure Eevee is the active render engine
                if context.scene.render.engine not in {'BLENDER_EEVEE', 'BLENDER_EEVEE_NEXT'}:
                    self.report({'WARNING'}, "Render engine is not set to Eevee or Eevee Next.")
                    return {'CANCELLED'}

                # Set World Settings
                world = context.scene.world
                if world is None:
                    world = bpy.data.worlds.new(name="World")
                    context.scene.world = world
                world.use_sun_shadow = True
                world.use_sun_shadow_jitter = True
                world.sun_shadow_maximum_resolution = 0.001
                world.probe_resolution = '2048'

                # Set Eevee Render Settings
                eevee = context.scene.eevee
                eevee.taa_samples = 32
                eevee.use_shadow_jitter_viewport = True
                eevee.use_shadows = True
                eevee.shadow_ray_count = 2
                eevee.use_raytracing = True
                eevee.ray_tracing_method = 'SCREEN'

                # Ensure ray_tracing_options exists (Blender 4.2+)
                if hasattr(eevee, 'ray_tracing_options'):
                    eevee.ray_tracing_options.resolution_scale = '1'
                    eevee.ray_tracing_options.trace_max_roughness = 1.0
                    eevee.ray_tracing_options.screen_trace_quality = 1.0
                    eevee.ray_tracing_options.screen_trace_thickness = 0.2
                else:
                    self.report({'WARNING'}, "Eevee ray_tracing_options not found. Settings skipped.")

                eevee.fast_gi_method = 'GLOBAL_ILLUMINATION'
                eevee.fast_gi_resolution = '1'
                eevee.fast_gi_ray_count = 4

                self.report({'INFO'}, "Switched to Rendered Shading: Eevee Next + Optimal Settings.")
            else:
                if blender_version >= (4, 2, 0):
                    self.report({'INFO'}, "Additional settings not applied. Add-on option is disabled.")
                else:
                    self.report({'INFO'}, "Additional settings not applied. Blender version is below 4.2.")

            # Redraw the window to apply changes
            bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
        else:
            self.report({'WARNING'}, "Active area is not a 3D Viewport.")
            return {'CANCELLED'}

        return {'FINISHED'}

class SetCyclesRenderedView(BaseOperator):
    """Switch to Rendered Shading and set the Render Engine to Cycles"""    
    bl_idname = "shading_plus.set_cycles_rendered_view"
    bl_label = "Cycles Rendered View"

    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':
            space = area.spaces.active
            context.scene.render.engine = 'CYCLES'
            space.shading.type = 'RENDERED'
            self.report({'INFO'}, "Switched to Rendered Shading: Cycles")
            bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
        return {'FINISHED'}

class SetWorkbenchRenderedView(BaseOperator):
    """Switch to Rendered Shading, set the Render Engine to Workbench, and retrieves the preset from Solid Shading"""
    bl_idname = "shading_plus.set_workbench_rendered_view"
    bl_label = "Workbench Rendered View"

    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':  # Ensure it's a 3D Viewport

            # Get the active space (viewport shading settings)
            space = area.spaces.active

            # Set the render engine to Workbench
            context.scene.render.engine = 'BLENDER_WORKBENCH'

            # Switch to Rendered shading mode with Workbench for the viewport
            space.shading.type = 'RENDERED'

            # Get the current shading preset
            shading_props = context.scene.shading_plus
            preset = getattr(shading_props, 'shading_preset', 'DEFAULT')
            print(f"Applying preset '{preset}' to Workbench Rendered view")

            # Apply the preset to the Workbench shading
            workbench_shading = context.scene.display.shading
            apply_preset_to_shading(preset, workbench_shading, shading_props)
        

            self.report({'INFO'}, "Switched to Rendered Shading: Workbench (Solid Shading's engine)")

            # Redraw the viewport to apply the changes
            bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
            print("Viewport updated")

        return {'FINISHED'}

class SetMaterialPreview(BaseOperator):
    """Switch to Material Preview (= EEVEE/EEVEE next with some settings off)"""
    bl_idname = "shading_plus.set_material_preview"
    bl_label = "Material Preview"

    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':
            space = area.spaces.active
            if context.scene.render.engine == 'BLENDER_WORKBENCH':
                context.scene.render.engine = 'CYCLES'  # or 'BLENDER_EEVEE' based on your preference
            space.shading.type = 'MATERIAL'
            self.report({'INFO'}, "Switched to Material Shading")
            bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
        return {'FINISHED'}

class SetHDRI(bpy.types.Operator):
    """Set HDRI Option"""
    bl_idname = "shading_plus.set_hdri_option"
    bl_label = "Set HDRI Option"
    bl_options = {'REGISTER', 'UNDO'}

    option: EnumProperty(
        name="HDRI Option",
        items=[
            ('BUILTIN', "Built-in HDRI", ""),
            ('WORLD', "World HDRI", ""),
        ]
    )

    def execute(self, context):
        # Ensure we have a valid 3D View context
        area = context.area
        if area and area.type == 'VIEW_3D':
            space = context.space_data
            shading_type = space.shading.type
            if shading_type == 'RENDERED':
                # In Rendered mode, set use_scene_world_render
                if self.option == 'WORLD':
                    space.shading.use_scene_world_render = True
                else:
                    space.shading.use_scene_world_render = False
            else:
                # In other modes, set use_scene_world
                if self.option == 'WORLD':
                    space.shading.use_scene_world = True
                else:
                    space.shading.use_scene_world = False
            return {'FINISHED'}
        else:
            self.report({'WARNING'}, "Active area is not a 3D Viewport.")
            return {'CANCELLED'}





class SyncShadowSettings(bpy.types.Operator):
    """Sync shadow settings across scenes with Shadows enabled in Solid Shading"""
    bl_idname = "shading_plus.sync_shadow_settings"
    bl_label = "Sync Shadow Settings"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        # Get current scene shadow settings
        current_display = context.scene.display
        light_direction = current_display.light_direction
        shadow_shift = current_display.shadow_shift
        shadow_focus = current_display.shadow_focus

        # Apply settings to all scenes
        for scene in bpy.data.scenes:
            scene.display.light_direction = light_direction
            scene.display.shadow_shift = shadow_shift
            scene.display.shadow_focus = shadow_focus

        self.report({'INFO'}, "Shadow settings applied to all scenes.")
        return {'FINISHED'}




# REGISTER/UNREGISTER
# __________________________________________________________________

classes = (
    UserShadingSettings,
    ShadingPlusProperties,
    SetCyclesRenderedView,
    SetEeveeRenderedView,
    SetMaterialPreview,
    SetWorkbenchRenderedView,
    SetHDRI,
    SaveUser1Preset,
    SyncShadowSettings,
)

def register():
    for cls in classes:
        try:
            bpy.utils.unregister_class(cls)
        except RuntimeError:
            pass
        bpy.utils.register_class(cls)
    
    bpy.types.Scene.shading_plus = PointerProperty(type=ShadingPlusProperties)


def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.shading_plus
    
    
if __name__ == "__main__":
    register()
